<?php

namespace Drupal\displayname\Element;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\FormElement;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\Template\Attribute;

/**
 * Provides a display name render element.
 *
 * @RenderElement("display_name")
 */
class DisplayName extends FormElement implements TrustedCallbackInterface {

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['preRender'];
  }

  /**
   * Returns the element properties for this element.
   *
   * @return array
   *   An array of element properties. See
   *   \Drupal\Core\Render\ElementInfoManagerInterface::getInfo() for
   *   documentation of the standard properties of all elements, and the
   *   return value format.
   */
  public function getInfo() {
    $parts = _displayname_translations();
    $field_settings = \Drupal::service('plugin.manager.field.field_type')
      ->getDefaultFieldSettings('display_name');

    return [
      '#input' => TRUE,
      '#process' => ['displayname_element_expand'],
      '#pre_render' => [[__CLASS__, 'preRender']],
      '#element_validate' => ['displayname_element_validate'],
      '#theme_wrappers' => ['form_element'],
      '#show_component_required_marker' => 1,
      '#default_value' => [
        'title' => '',
        'firstname' => '',
        'middle' => '',
        'lastname' => '',
        'full' => '',
        'nickname' => '',
      ],
      '#minimum_components' => $field_settings['minimum_components'],
      '#allow_lastname_or_firstname' => $field_settings['allow_lastname_or_firstname'],
      '#components' => [
        'title' => [
          'type' => $field_settings['field_type']['title'],
          'title' => $parts['title'],
          'title_display' => 'description',
          'size' => $field_settings['size']['title'],
          'maxlength' => $field_settings['max_length']['title'],
          'options' => $field_settings['title_options'],
          'autocomplete' => FALSE,
        ],
        'firstname' => [
          'type' => 'textfield',
          'title' => $parts['firstname'],
          'title_display' => 'description',
          'size' => $field_settings['size']['firstname'],
          'maxlength' => $field_settings['max_length']['firstname'],
          'autocomplete' => FALSE,
        ],
        'middle' => [
          'type' => 'textfield',
          'title' => $parts['middle'],
          'title_display' => 'description',
          'size' => $field_settings['size']['middle'],
          'maxlength' => $field_settings['max_length']['middle'],
          'autocomplete' => FALSE,
        ],
        'lastname' => [
          'type' => 'textfield',
          'title' => $parts['lastname'],
          'title_display' => 'description',
          'size' => $field_settings['size']['lastname'],
          'maxlength' => $field_settings['max_length']['lastname'],
          'autocomplete' => FALSE,
        ],
        'full' => [
          'type' => 'textfield',
          'title' => $parts['full'],
          'title_display' => 'description',
          'size' => $field_settings['size']['full'],
          'maxlength' => $field_settings['max_length']['full'],
          'autocomplete' => FALSE,
        ],
        'nickname' => [
          'type' => 'textfield',
          'title' => $parts['nickname'],
          'title_display' => 'description',
          'size' => $field_settings['size']['nickname'],
          'maxlength' => $field_settings['max_length']['nickname'],
          'autocomplete' => FALSE,
        ],
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function valueCallback(&$element, $input, FormStateInterface $form_state) {
    $value = [
      'title' => '',
      'firstname' => '',
      'middle' => '',
      'lastname' => '',
      'full' => '',
      'nickname' => '',
    ];
    if ($input === FALSE) {
      $element += ['#default_value' => []];
      return $element['#default_value'] + $value;
    }
    // Throw out all invalid array keys; we only allow pass1 and pass2.
    foreach ($value as $allowed_key => $default) {
      // These should be strings, but allow other scalars since they might be
      // valid input in programmatic form submissions. Any nested array values
      // are ignored.
      if (isset($input[$allowed_key]) && is_scalar($input[$allowed_key])) {
        $value[$allowed_key] = (string) $input[$allowed_key];
      }
    }
    return $value;
  }

  /**
   * This function themes the element and controls the title display.
   */
  public static function preRender($element) {
    $layouts = displayname_widget_layouts();
    $layout = $layouts['stacked'];
    if (!empty($element['#widget_layout']) && isset($layouts[$element['#widget_layout']])) {
      $layout = $layouts[$element['#widget_layout']];
    }

    if (!empty($layout['library'])) {
      if (!isset($element['#attached']['library'])) {
        $element['#attached']['library'] = [];
      }
      $element['#attached']['library'] += $layout['library'];
    }
    $attributes = new Attribute($layout['wrapper_attributes']);
    $element['_displayname'] = [
      '#prefix' => '<div' . $attributes . '>',
      '#suffix' => '</div>',
    ];

    foreach (_displayname_translations() as $key => $title) {
      if (isset($element[$key])) {
        $element['_displayname'][$key] = $element[$key];
        unset($element[$key]);
      }
    }

    return $element;
  }

}
